import('model.events.gui.GuiLocationChangedEvent')
import('model.Component')
import('model.toolbar.header.ToolbarHeader')
import('model.toolbar.header.Unlock')
import('model.toolbar.content.ToolbarContent')
import('model.toolbar.content.sections.Sections')
import('model.toolbar.content.sections.section.Section')
import('model.toolbar.content.sections.section.content.table.Table')

---@class Toolbar : Component
Toolbar = Component:extendAs("model.toolbar.Toolbar")

function Toolbar:create(parent)
    local instance = Component.create(self, parent, {
        type = "frame",
        direction = "vertical",
        style = "toolbar" })
    ToolbarHeader:create(instance)
    ToolbarContent:create(instance)
    instance:adjustSize()
    return instance
end

function Toolbar:new(parent, root)
    local instance = Toolbar:super(Component:new(parent, root))
    instance:loadChildren()
    return instance
end

function Toolbar:childrenClasses()
    return { ToolbarHeader, ToolbarContent }
end

---@return ToolbarHeader
function Toolbar:header()
    return self:childOfType(ToolbarHeader)
end

---@return ToolbarContent
function Toolbar:content()
    return self:childOfType(ToolbarContent)
end

---@public
function Toolbar:centerOnScreen()
    self:root().force_auto_center()
end

function Toolbar:handleGuiChange(guiChangedEvent)
    if self:isOriginOf(guiChangedEvent) then
        if guiChangedEvent:isInstanceOf(GuiLocationChangedEvent) then
            self:keepWithinTheScreen()
            self:turnOffCenteringOnScreen()
        end
    else
        Component.handleGuiChange(self, guiChangedEvent)
    end
end

---@private
function Toolbar:keepWithinTheScreen()
    local location = self:location()
    if location.x < 0 then
        location.x = 0
    end
    if location.y < 0 then
        location.y = 0
    end
    self:setLocation(location)
end

---@private
function Toolbar:turnOffCenteringOnScreen()
    self:root().auto_center = false
end

---@private
---@param location GuiLocation
function Toolbar:setLocation(location)
    self:root().location = location
end

---@private
---@return GuiLocation
function Toolbar:location()
    return self:root().location
end

---@public
function Toolbar:collapse()
    self:content():hide()
end

---@public
function Toolbar:expand()
    self:content():show()
end

---@public
function Toolbar:addSection()
    self:content():sections():addSection()
    self:resizeTablesInUnlock()
end

---@public
---@return number
function Toolbar:sectionsCount()
    local content = self:content()
    local sectionList = content:sections()
    local count = sectionList:count()
    return count
end

function Toolbar:lock()
    self:content():root().style = "toolbar_content"
    self:trimTables()
    Component.lock(self)
end

function Toolbar:unlock()
    self:resizeTablesInUnlock()
    Component.unlock(self)
end

---@public
function Toolbar:adjustSize()
    if not self:isLocked() then
        self:resizeTablesInUnlock()
    end
end

---@private
function Toolbar:trimTables()
    local minimumWidth = 1
    if self:hasSectionWithLongerName() then
        minimumWidth = 2
    end
    self:resizeTables(minimumWidth, 1, 0)
end

---@private
---@return boolean
function Toolbar:hasSectionWithLongerName()
    for i, section in ipairs(self:content():sections():list()) do
        if section:name():len() > 3 then
            return true
        end
    end
    return false
end

---@private
function Toolbar:resizeTablesInUnlock()
    self:resizeTables(4, 1, 1)
end

function Toolbar:resizeTables(minimumWidth, minimumHeight, padding)
    local width = math.max(minimumWidth, self:lastOccupiedColumnIndex() + padding)
    self:adjustTables(width, minimumHeight, padding)
end

---@private
---@param width number
function Toolbar:adjustTables(width, minimumHeight, padding)
    for _, table in ipairs(self:tables()) do
        table:resize(width, minimumHeight, padding)
    end
end

---@return boolean
---@private
function Toolbar:isLocked()
    return self:childOfType(ToolbarHeader):childOfType(Unlock) ~= nil
end

---@public
---@return number
function Toolbar:lastOccupiedColumnIndex()
    local lastOccupiedColumnIndex = 0
    for _, table in ipairs(self:tables()) do
        lastOccupiedColumnIndex = math.max(lastOccupiedColumnIndex, table:lastOccupiedColumnIndex())
    end
    return lastOccupiedColumnIndex
end

---@private
---@return Table[]
function Toolbar:tables()
    local tables = {}
    for _, section in pairs(self:content():sections():list()) do
        table.insert(tables, section:content():table())
    end
    return tables
end
