import('model.events.gui.GuiClickEvent')
import('model.events.CursorStackChanged')
import('model.events.InventoryChanged')
import('model.toolbar.content.sections.section.content.table.Slot')

---@class Item : Slot
Item = Slot:extendAs("model.toolbar.content.sections.section.content.table.slots.Item")

function Item:create(parent, itemName)
    local instance = Component.create(self, parent, {
        type = "sprite-button",
        elem_tooltip = { type = "item", name = itemName },
        sprite = "item/" .. assert(itemName),
        style = "toolbar_content_sections_section_content_table_row_slot_item",
        mouse_button_filter = { "left", "middle" },
    })
    instance:updateCount()
    return instance
end

function Item:new(parent, root)
    return Item:super(Slot:new(parent, root))
end

function Item:handleGuiClick(guiClickEvent)
    local player = self:player()
    if guiClickEvent:isLeftClick() and guiClickEvent:withModifier() then
        if not player.character then
            player.play_sound { path = "utility/cannot_build" }
            player.create_local_flying_text { text = { "gui-map-editor.title" }, create_at_cursor = true }
        elseif self:hasRecipe() then
            local recipe = self:recipe()
            if not recipe.enabled then
                player.play_sound { path = "utility/cannot_build" }
                player.create_local_flying_text { text = { "description.recipe-not-researched" }, create_at_cursor = true }
            elseif player.force.get_hand_crafting_disabled_for_recipe(recipe) then
                player.create_local_flying_text { text = { "recipe-not-craftable-in-hand" }, create_at_cursor = true }
                player.play_sound { path = "utility/cannot_build" }
            elseif player.get_craftable_count(recipe) == 0 then
                player.create_local_flying_text { text = { "not-enough-ingredients" }, create_at_cursor = true }
                player.play_sound { path = "utility/cannot_build" }
            else
                player.play_sound { path = "utility/inventory_click" }
                local count = 1
                if guiClickEvent:withCtrl() then
                    count = 1
                elseif guiClickEvent:withAlt() then
                    count = 5
                elseif guiClickEvent:withShift() then
                    count = player.get_craftable_count(recipe)
                end
                player.create_local_flying_text { text = tostring(count), create_at_cursor = true }
                player.begin_crafting { count = count, recipe = recipe.name }
            end
        else
            player.play_sound { path = "utility/cannot_build" }
            player.create_local_flying_text { text = { "entity-status.no-recipe" }, create_at_cursor = true }
        end
    elseif guiClickEvent:isLeftClick() then
        player.play_sound { path = "utility/inventory_click" }
        local itemName = self:itemName()
        if self:inventory():hasItem(itemName) then
            self:inventory():pick(itemName)
        else
            self:cursor():pickGhost(itemName)
        end
        self:rememberPick()
    elseif guiClickEvent:isMiddleClick() then
        self:clear()
    end
end

---@private
---@return boolean
function Item:hasRecipe()
    return self:recipe() ~= nil
end

---@private
---@return LuaRecipe
function Item:recipe()
    local recipePrototype = self:recipePrototype()
    if recipePrototype then
        return self:player().force.recipes[recipePrototype.name]
    end
end

---@private
---@return LuaRecipePrototype
function Item:recipePrototype()
    local recipes = game.get_filtered_recipe_prototypes(
            { {
                  filter = "has-product-item",
                  elem_filters = { { filter = "name", name = self:itemName() } }
              } })
    for _, recipe in pairs(recipes) do
        return recipe
    end
    return nil
end

---@param event Event
function Item:handle(event)
    if event:isInstanceOf(CursorStackChanged) then
        self:highlightHeldItem()
        self:updateCount()
    elseif event:isInstanceOf(InventoryChanged) then
        self:updateCount()
    end
end

---@private
function Item:highlightHeldItem()
    local cursor = self:cursor()
    if cursor:holdsStack() and cursor:itemName() == self:itemName() then
        self:root().style = "toolbar_content_sections_section_content_table_row_slot_item_selected"
    elseif cursor:holdsGhost() and cursor:itemName() == self:itemName() then
        self:root().style = "toolbar_content_sections_section_content_table_row_slot_item_selected"
    else
        self:root().style = "toolbar_content_sections_section_content_table_row_slot_item"
    end
end

---@private
function Item:updateCount()
    local count = self:inventory():countOf(self:itemName())
    local cursor = self:cursor()
    if cursor:holdsStack() and cursor:itemName() == self:itemName() then
        count = count + cursor:stackCount()
    end
    self:root().number = count > 0 and count or nil
end

function Item:itemName()
    return self:root().elem_tooltip.name
end
